@@ -9,14 +9,17 @@ By default,
 
 """
 
-from abc import ABC, abstractmethod, abstractproperty
+from abc import ABC, abstractmethod
 from enum import Enum
-from typing import Any, ClassVar, Union
+from typing import Any, Mapping, Union
+from contextlib import contextmanager
+
 
 class ContextError(Exception):
     """Any errors related to contexts"""
 
-class ContextBase(ABC): # pragma: no cover
+
+class ContextBase(ABC):  # pragma: no cover
     """The context abstract class, defining how
     the Reference objects are evaluated
 
@@ -28,16 +31,33 @@ class ContextBase(ABC): # pragma: no cover
         Since we could do `f[f.A]`.
     """
 
+    def __init__(self, meta: Mapping[str, Any] = None):
+        """Meta data is carring down"""
+        self.meta = meta or {}
+
+    def eval_symbolic(self, data: Any) -> Any:
+        return data
+
     @abstractmethod
-    def getattr(self, parent: Any, ref: str, is_direct: bool = False) -> Any:
+    def getattr(self, parent: Any, ref: str, level: int) -> Any:
         """Defines how `f.A` is evaluated"""
 
     @abstractmethod
-    def getitem(self, parent: Any, ref: Any, is_direct: bool = False) -> Any:
+    def getitem(self, parent: Any, ref: Any, level: int) -> Any:
         """Defines how `f[item]` is evaluated"""
 
     def __repr__(self) -> str:
-        return f'<{self.__class__.__name__} @ {hex(id(self))}>'
+        return f"<{self.__class__.__name__} @ {hex(id(self))}>"
+
+    @contextmanager
+    def with_meta(self, meta: Mapping[str, Any]):
+        if meta is None or meta is self.meta:
+            yield
+        else:
+            self_meta = self.meta.copy()
+            self.meta = meta
+            yield
+            self.meta = self_meta
 
     @property
     def ref(self) -> "ContextBase":
@@ -56,9 +76,6 @@ class ContextBase(ABC): # pragma: no cover
         """The context to evaluate `**kwargs` passed to a function"""
         return self
 
-    @abstractproperty
-    def name(self) -> str:
-        """The name of the context"""
 
 class ContextSelect(ContextBase):
     """Context used in a select context
@@ -68,60 +85,58 @@ class ContextSelect(ContextBase):
     - `f[ref]` works as a shortcut of `ref`. However, `ref` is needed to be
         evaluated by a context returned by `getref`
     """
-    name: ClassVar[str] = 'select'
 
-    def getattr(self, parent: Any, ref: str, is_direct: bool = False) -> str:
+    def getattr(self, parent: Any, ref: str, level: int) -> str:
         """Get the `ref` directly, regardless of `data`"""
         return ref
 
-    def getitem(self, parent: Any, ref: Any, is_direct: bool = False) -> Any:
+    def getitem(self, parent: Any, ref: Any, level: int) -> Any:
         """Get the `ref` directly, which is already evaluated by `f[ref]`"""
         return ref
 
+
 class ContextEval(ContextBase):
     """Context used in a data-evaluation context
 
     In this kind of context, the expression is evaluated as-is. That is,
     `f.A` is evaluated as `f.A` and `f[item]` is evaluated as `f[item]`
     """
-    name: ClassVar[str] = 'eval'
 
-    def getattr(self, parent: Any, ref: str, is_direct: bool = False) -> Any:
+    def getattr(self, parent: Any, ref: str, level: int) -> Any:
         """How to evaluate `f.A`"""
         return getattr(parent, ref)
 
-    def getitem(self, parent: Any, ref: Any, is_direct: bool = False) -> Any:
+    def getitem(self, parent: Any, ref: Any, level: int) -> Any:
         """How to evaluate `f[item]`"""
         return parent[ref]
 
+
 class ContextPending(ContextBase):
     """Pending context"""
-    name: ClassVar[str] = 'pending'
 
-    def getattr(self, parent: Any, ref: str, is_direct: bool = False) -> str:
+    def getattr(self, parent: Any, ref: str, level: int) -> str:
         """Get the `ref` directly, regardless of `data`"""
         raise NotImplementedError(
-            'Pending context cannot be used to evaluate.'
+            "Pending context cannot be used to evaluate."
         )
 
-    def getitem(self, parent: Any, ref: Any, is_direct: bool = False) -> Any:
+    def getitem(self, parent: Any, ref: Any, level: int) -> Any:
         """Get the `ref` directly, which is already evaluated by `f[ref]`"""
         raise NotImplementedError(
-            'Pending context cannot be used to evaluate.'
+            "Pending context cannot be used to evaluate."
         )
 
 
 class ContextMixed(ContextBase):
     """A mixed context, where the `*args` are evaluated with `ContextSelect`
     and `**args` are evaluated with `ContextEval`."""
-    name: ClassVar[str] = 'mixed'
 
-    def getattr(self, parent: Any, ref: str, is_direct: bool = False) -> None:
+    def getattr(self, parent: Any, ref: str, level: int) -> None:
         raise NotImplementedError(
             "Mixed context should be used via `.args` or `.kwargs`"
         )
 
-    def getitem(self, parent: Any, ref: Any, is_direct: bool = False) -> None:
+    def getitem(self, parent: Any, ref: Any, level: int) -> None:
         raise NotImplementedError(
             "Mixed context should be used via `.args` or `.kwargs`"
         )
@@ -134,6 +149,7 @@ class ContextMixed(ContextBase):
     def kwargs(self):
         return ContextEval()
 
+
 class Context(Enum):
     """Context to solve f.A and f['A']
 
@@ -146,10 +162,12 @@ class Context(Enum):
         For *args, used select-based;
         for **kwargs, use evaluation-based.
     """
+
     UNSET = None
     PENDING = ContextPending()
     SELECT = ContextSelect()
     EVAL = ContextEval()
     MIXED = ContextMixed()
 
+
 ContextAnnoType = Union[Context, ContextBase]
